// copyright

/**
 * Configuration is a data structure that handles the
 * configuration-- global data, program arguments, etc.--
 * of the CSim Car Simulator.
 *
 * @author Steven Bergom
 */


#ifndef _CSIM_CONF_H
#define _CSIM_CONF_H

extern void errorHandler(int err_no);

typedef double (*rand_func)(void);
typedef enum{CEBadFileName=1, CEBadArgument} ConfigurationError;
typedef enum{ET2d, ET3d} EnvironmentType;

class Configuration
{
  /**
   * Public functions
   */
 public:
  Configuration();
  Configuration(int argc, char **argv);
  // Generates the configuration file from the program arguments
  ~Configuration();

  ConfigurationError readConfiguration(int argc, char **argv);

  /**
   * Protected functions
   */
 protected:

  bool readConfigFile(FILE *file_name);

  void printArguments();
  // Prints the arguments acceptable to CSim and exits

  void printCopyright();
  // Prints the copyright info to stderr

  /**
   * Private functions
   */
 private:
  Configuration(const Configuration &cnf) {};

  /**
   * Public data members
   */
 public:
  Random          rs;               // The source of all randomness
  char           *program_name;     // Name of the plug-in, NULL if generic
  int             num_dimensions;   // Number of "dimensions" in the model
  Model          *environment;      // Information about the environment
  EnvironmentType e_type;           // 2d or 3d
  Graph          *path;             // The path that the vehicle follows
  ModelManager   *m_manager;        // Manages the dynamics of the model
  double          time_step;        // Time between "frames"
  unsigned long   num_its;          // Number of iterations to perform
  bool            verbose;          // Status messages should be suppressed
  double_vector   begin_state;      // The starting state of the model
  Acceptor        accept_state;     // The ending state of the model
  ostream        *out;              // Where to send output
  bool            profiling;        // Whether to print profiling data

  /**
   * Public data members
   */
 protected:

  /**
   * Public data members
   */
 private:
};

#endif // _CSIM_CONF_H
