// copyright

/**
 * These classes contain various types of ending
 * conditions to search for, including a range of
 * values, specific value, or any value.
 *
 * @author Steven Bergom
 */

#ifndef _CSIM_ENDCOND_H
#define _CSIM_ENDCOND_H

#define EPSILON 10e-10

typedef enum{STNoSet=0, STNone, STSphere, STGravity, STCenter} SkewType;

class EndCondition
{
  /**
   * Public functions
   */
 public:
  EndCondition() {}
  virtual ~EndCondition() {}

  virtual bool accept(double val)=0;
  virtual double center()=0;

  /**
   * Protected functions
   */
 protected:

  /**
   * Private functions
   */
 private:

  /**
   * Public data members
   */
 public:

  /**
   * Public data members
   */
 protected:

  /**
   * Public data members
   */
  private:
};

class ECRange : public EndCondition
{
  /**
   * Public functions
   */
 public:
  ECRange();
  ECRange(double s_val, double e_val) { r1 = s_val; r2 = e_val; }
  // Assert: s_val < e_val
  virtual ~ECRange() {}

  virtual bool accept(double val) { return (val > r1) && (val < r2); }
  virtual double center() { return (r1+r2)/2.0; }

  /**
   * Protected functions
   */
 protected:

  /**
   * Private functions
   */
 private:

  /**
   * Public data members
   */
 public:

  /**
   * Public data members
   */
 protected:
  double r1, r2;

  /**
   * Public data members
   */
  private:
};

class ECVal : public EndCondition
{
  /**
   * Public functions
   */
 public:
  ECVal() {}
  ECVal(double val) { r_val = val; }
  virtual ~ECVal() {}

  virtual bool accept(double val) { return (val > r_val-EPSILON) && 
				      (val < r_val+EPSILON); }
  virtual double center() { return r_val; }

  /**
   * Protected functions
   */
 protected:

  /**
   * Private functions
   */
 private:

  /**
   * Public data members
   */
 public:

  /**
   * Public data members
   */
 protected:
  double r_val;

  /**
   * Public data members
   */
  private:
};

class ECAny : public EndCondition
{
  /**
   * Public functions
   */
 public:
  ECAny() {}
  virtual ~ECAny() {}

  virtual bool accept(double val) { return TRUE; }
  virtual double center() { return -1; }

  /**
   * Protected functions
   */
 protected:

  /**
   * Private functions
   */
 private:

  /**
   * Public data members
   */
 public:

  /**
   * Public data members
   */
 protected:

  /**
   * Public data members
   */
  private:
};

class Acceptor
{
  /**
   * Public functions
   */
 public:
  Acceptor() {}
  virtual ~Acceptor();

  virtual bool accept(double_vector conf);
  virtual void addCondition(EndCondition *ec) { conditions.push_back(ec); }
  virtual SkewType setSkewType(SkewType st=STNoSet);
  virtual double_vector skewConf(double_vector &dv);

  /**
   * Protected functions
   */
 protected:

  /**
   * Private functions
   */
 private:

  /**
   * Public data members
   */
 public:

  /**
   * Public data members
   */
 protected:
  vector< EndCondition* > conditions;

  /**
   * Public data members
   */
  private:
  SkewType        skew_type;        // Remeber the skew type for queries
  double_vector   center_goal;
  long            skew_count;
};

#undef EPSILON

#endif // _CSIM_ENDCOND_H
