// copyright

/**
 * The Graph and Node classes provide an efficient mechanism
 * for storing a Rapidly Exploring Random Tree (RRT) structure
 * and to access that information.
 *
 * @author Steven Bergom
 */

#ifndef _CSIM_GRAPH_H
#define _CSIM_GRAPH_H

inline double SQR(double d1);

class GraphNode
{
  friend class Graph;

  /**
   * Public functions
   */
 public:
  GraphNode();
  GraphNode(const GraphNode &gn);
  GraphNode(const double_vector &dv);
  ~GraphNode();

  void setData(const double_vector &dv);
  double_vector getData();
  unsigned long getSize();

  unsigned long getParent();
  void setParent(GraphNode &gn);

  friend ostream& operator<<(ostream &out, const GraphNode &gn);

  /**
   * Protected functions
   */
 protected:

  /**
   * Private functions
   */
 private:

  /**
   * Public data members
   */
 public:
  
  /**
   * Public data members
   */
 protected:
  double_vector data;
  unsigned long parent;
  unsigned long graph_id;

  /**
   * Public data members
   */
 private:
};

class Graph
{
  /**
   * Public functions
   */
 public:
  Graph(GraphNode &gn);
  ~Graph();

  void insert(GraphNode _parent, GraphNode child);
  // Assert: _parent is in the current graph

  unsigned long getNumberNodes() { return current_end+1; }

  GraphNode operator[] (unsigned long index) { return children[index]; }

  vector< GraphNode >* getAllChildren() { return &children; }
  // Warning: Should not be used by user!

  vector< GraphNode* > getNodeLine(const GraphNode &gn);
  // Retrieves the line of nodes from the head to gn

  friend ostream& operator<<(ostream &out, Graph &g);

  /**
   * Protected functions
   */
 protected:

  /**
   * Private functions
   */
 private:
  // We make the copy-constructor and assignment private
  // to avoid any bad memory errors since this graph could
  // easily become quite large.
  Graph(const Graph &g) {}
  operator=(const Graph &g1) {}

  /**
   * Public data members
   */
 public:

  /**
   * Public data members
   */
 protected:
  vector< GraphNode > children;
  unsigned long current_end;

  /**
   * Public data members
   */
 private:
};

#endif // _CSIM_GRAPH_H
