// copyright

/**
 * Model provides an efficient structure for storing and
 * accessing a 3-dimesional object.  It also allows the
 * manipulation of the model and intersection with other
 * models.
 *
 * @author Steven Bergom
 */

#include "polygon.h"

#ifndef _CSIM_MODEL_H
#define _CSIM_MODEL_H
 
typedef enum{
  MFTOpenInventor, 
    MFTVrml, 
    MFTVclipNative, 
    MFTCsimNative
    } ModelFileType;

class Model
{
  /**
   * Public functions
   */
 public:
  Model() {}
  virtual ~Model() {}

  virtual int intersect(const double_vector &dv)=0; 

  virtual void optimize()=0;
  // Optimizes the model structure for future optimizations
  // Warning: Should only be called once

  virtual void describe(ostream &os)=0;

  virtual Polygon *getRobot(){}

  /**
   * Protected functions
   */
 protected:

  /**
   * Private functions
   */
 private:

  /**
   * Public data members
   */
 public:

  /**
   * Public data members
   */
 protected:

  /**
   * Public data members
   */
 private:
};

class Model2d : public Model
{
  /**
   * Public functions
   */
 public:
  Model2d();
  Model2d(const Model2d &m);
  Model2d(const char *file_name, ModelFileType mft);
  virtual ~Model2d();

  virtual int intersect(const double_vector &dv);

  void optimize() {}
  // Optimizes the model structure for future optimizations
  // Warning: Should only be called once

  virtual void describe(ostream &os);

  virtual Polygon *getRobot() { return robot; }

  /**
   * Protected functions
   */
 protected:

  /**
   * Private functions
   */
 private:

  /**
   * Public data members
   */
 public:

  /**
   * Public data members
   */
 protected:

  vector< Polygon* > world;
  Polygon *robot;

  /**
   * Public data members
   */
 private:
};

class Model3d : public Model
{
  /**
   * Public functions
   */
 public:
  Model3d();
  Model3d(const Model3d &m);
  Model3d(const char *file_name, ModelFileType mft);
  virtual ~Model3d();

  virtual int intersect(const double_vector &dv);

  void optimize();
  // Optimizes the model structure for future optimizations
  // Warning: Should only be called once

  virtual void describe(ostream &os) {}

  /**
   * Protected functions
   */
 protected:

  /**
   * Private functions
   */
 private:

  /**
   * Public data members
   */
 public:

  /**
   * Public data members
   */
 protected:

  /**
   * Public data members
   */
 private:
};

#endif // _CSIM_MODEL_H
