%option noyywrap

%{
#include "csim.h"
#include "csim_parse.h"

int yyline = 1;

char string_buf[256];
char *string_buf_ptr;

%}

yyDIGIT [0-9]
yyINTNUM {yyDIGIT}+
yyCHAR [a-zA-Z]
yyCHARNUM ({yyCHAR}*{yyDIGIT}*_*)*

%x slashslash str
%%

\n      { yyline++; }

\{      { return TOK_LBRACE; }

\}      { return TOK_RBRACE; }


"//"                     BEGIN(slashslash);
<slashslash>.*[^\n]
<slashslash>[\n]         yyline++; BEGIN(INITIAL);

"\""/[^\n]*\"           string_buf_ptr = string_buf; BEGIN(str);
<str>"\""               {
                         BEGIN(INITIAL); 
                         *string_buf_ptr = '\0';
                         yylval.strVal = string_buf;
                         return TOK_STRING;
                         }
<str>"\n"             *string_buf_ptr = '\0'; BEGIN(INITIAL);
<str>"\\\""           *string_buf_ptr++ = '\"';
<str>"\\t"            *string_buf_ptr++ = '\t';
<str>"\\n"            *string_buf_ptr++ = '\n';
<str>\\(.|\n)         *string_buf_ptr++ = yytext[1];
<str><<EOF>>          BEGIN(INITIAL); return TOK_SYNTAX_ERROR;
<str>[^\\\n\"]+       {
    char *yptr = yytext;
    while (*yptr) *string_buf_ptr++ = *yptr++;
}

";"      { return TOK_SEMI; }

(({yyINTNUM}("."{yyINTNUM})?)|({yyINTNUM}?"."{yyINTNUM}))((e|E)(\+|\-)?{yyINTNUM})? {
    yylval.floatVal = (float)atof(yytext);
    return TOK_FLOAT;
}

"model"  { return TOK_MODEL; }

"world"  { return TOK_WORLD; }

"time_lapse" { return TOK_TIME_LAPSE; }

"iterations" { return TOK_ITERATIONS; }

"begin_state" { return TOK_BEGIN_STATE; }

"end_state"   { return TOK_END_STATE; }

"state_names" { return TOK_STATE_NAMES; }

"range" { return TOK_RANGE; }

"dimensions" { return TOK_DIM; }

\( { return TOK_LPAREN; }

\) { return TOK_RPAREN; }

\* { return TOK_VARIABLE; }

\, { return TOK_COMMA; }

. { yylval.strVal = yytext; }

%%
 
