#include "csim.h"

#define BUF_SIZE 100

Model2d::Model2d()
{
}

Model2d::Model2d(const Model2d &m)
{
  for (int i=0; i<m.world.size(); i++)
  {
    world.push_back( new Polygon(*m.world[i]) );
  }
}

Model2d::Model2d(const char *file_name, ModelFileType mft)
{
  // Assuming MFTCsimNative file type
  fstream infile(file_name, ios::in);

  if (infile)
  {
    char comment_buf[BUF_SIZE];
    char cmd;
    
    while (!infile.eof())
    {
      infile >> cmd;
      switch (cmd)
      {
      case '#':
	infile.getline(comment_buf, BUF_SIZE);
	break;

      case 'p':
	{
	  int num_verts;
	  Polygon *p = new Polygon();
	  infile >> num_verts;

	  for (int i=0; i<num_verts; i++)
	  {
	    double a, b;
	    Point new_point;
	    infile >> a >> b;
	    new_point.set(a, b);
	    p->addVert(new_point);
	  } // end for
	  world.push_back(p);
	}
	break;

      case 'r':
	{
	  // This is the robot description
	  int num_verts;
	  infile >> num_verts;

	  if (robot != NULL)
	  {
	    delete robot;
	  }

	  robot = new Polygon();
	  
	  for (int i=0; i<num_verts; i++)
	  {
	    double a, b;
	    Point new_point;
	    infile >> a >> b;
	    new_point.set(a, b);
	    robot->addVert(new_point);
	  } // end for

	}
	break;

      default:
	cerr << "Unrecognized command in " << file_name << " : " << cmd;
	break;
      } // end switch(cmd)
    } // end while(infile)

    if (robot == NULL)
    {
      // Create a point robot
      Point pt(0,0);
      robot = new Polygon();
      robot->addVert(pt);
    }
  }
  else
  {
    cerr << file_name << ": File not found for reading.\n"
	 << "Using empty world.\n";
  }
}

Model2d::~Model2d()
{
  int ws = world.size();
  for (int i=0; i<ws; i++)
  {
    delete world[i];
  }
}

int Model2d::intersect(const double_vector &dv)
{
  Polygon *trans_robot;

  if (dv[0] > 1 || dv[0] < 0 ||
      dv[1] > 1 || dv[1] < 0)
  {
    return 1;
  }

  trans_robot = robot->transform(dv);

  for (int i=0; i<world.size(); i++)
  {
    if ( world[i]->intersect( trans_robot ) )
    {
      return 1;
    }
  }
  return 0;
}

void Model2d::describe(ostream &os)
{
  int world_size = world.size();

  for (int i=0; i<world_size; i++)
  {
    world[i]->describe(os);
  }
}

//---------------------------------------------------------

Model3d::Model3d()
{
}

Model3d::Model3d(const Model3d &m)
{
}

Model3d::Model3d(const char *file_name, ModelFileType mft)
{
}

Model3d::~Model3d()
{
}


int Model3d::intersect(const double_vector &dv)
{
}

void Model3d::optimize()
{  
}  
  

