%{
#include <iostream.h>
#include "csim.h"
#define prt(x) cout << x << endl

extern int verbosity;
extern int yylex();
extern void yyerror(char *s);
%}

%start input 

%union {
   char* strVal;
   int   intVal;
   float floatVal;
   double_vector *dvecVal;
   EndCondition *ecVal;
};

/* Tokens */
%token TOK_CSIM
%token TOK_STRING , TOK_FLOAT
%token TOK_MODEL , TOK_WORLD
%token TOK_BEGIN_STATE , TOK_END_STATE , TOK_STATE_NAMES
%token TOK_ITERATIONS , TOK_TIME_LAPSE
%token TOK_RBRACE , TOK_LBRACE , TOK_COMMA , TOK_SEMI
%token TOK_LPAREN , TOK_RPAREN
%token TOK_RANGE , TOK_VARIABLE , TOK_DIM
%token TOK_SYNTAX_ERROR

%type <strVal> TOK_STRING
%type <ecVal>  EndStateChoice
%type <dvecVal> FloatList
%type <floatVal> TOK_FLOAT

%%

input:
	/* emtpy */
	|
	WorldDescription ModelDescription StateDescription ExtraInfo
	;

WorldDescription:
	TOK_WORLD TOK_STRING TOK_SEMI
        {
          main_config.environment = new Model2d($2, MFTCsimNative);
        }
	;

ModelDescription:
	TOK_MODEL TOK_STRING TOK_SEMI
        {
          main_config.m_manager = new MMPlugInDL($2);
        }
	;

StateDescription:
	/* empty */
	|
	StateDescription StateDescriptionChoice
	;

StateDescriptionChoice:
	BeginState
	|
	EndState
	|
	StateNames
	;

BeginState:
	TOK_BEGIN_STATE TOK_LBRACE FloatList TOK_RBRACE
        {
          main_config.begin_state = *$3;
        }
	;

FloatList:
	/* empty */
        {
          $$ = new double_vector;
        }
	|
	FloatList TOK_FLOAT
        {
          $1->push_back((double)$2);
        }
 	;

EndState:
	TOK_END_STATE TOK_LBRACE EndStateList TOK_RBRACE
	;

EndStateList:
        /* empty */
        |
        EndStateList EndStateChoice
        {
          main_config.accept_state.addCondition($2);
        }
        ;

EndStateChoice:
        TOK_RANGE TOK_LPAREN TOK_FLOAT TOK_COMMA TOK_FLOAT TOK_RPAREN
        {
          $$ = new ECRange($3, $5);
        }
        |
        TOK_VARIABLE
        {
          $$ = new ECAny();
        }
        ;

StateNames: 
	TOK_STATE_NAMES TOK_LBRACE StringList TOK_RBRACE
 	;

ExtraInfo:
	/* empty */
	|
	ExtraInfo NumberIterations
	|
	ExtraInfo TimeLapse
        |
        ExtraInfo Dimensions
	;

NumberIterations:
	TOK_ITERATIONS TOK_FLOAT TOK_SEMI
        {
          main_config.num_its = (unsigned long)$2;
        }
	;

TimeLapse:
	TOK_TIME_LAPSE TOK_FLOAT TOK_SEMI
        {
	  main_config.time_step = (double)$2;
        }
	;

Dimensions:
	TOK_DIM TOK_FLOAT TOK_SEMI
        {
          main_config.num_dimensions = (int)$2;
        }
        ;

StringList:
	/* empty */
	{
	}
	|
	StringList TOK_STRING
	{
	}
	;

%%


