#include "csim.h"

void csim_main(int argc, char **argv)
{
  int from_csim, to_csim;
  int tmp_int = 1;
  CSimPluginSignal pgs;

  // Make sure that this is the right way to execute
  if (argc<4 || strcmp(argv[3], "-csim")!=0) 
  {
    fprintf(stderr, "%s: This is a CSim plugin and must be executed from CSim.\n", argv[0]);
    return;
  }

  from_csim = atoi(argv[1]);
  to_csim   = atoi(argv[2]);

  // Do any necessary initialization
  if (CSIM_PLUGIN.init_func != NULL) 
  {
    (*CSIM_PLUGIN.init_func)();
  }

  write(to_csim, (void *)&tmp_int, INT_SIZE);

  // Start the read-loop
  while (1) 
  {
    read(from_csim, (void *)&pgs, INT_SIZE);

    switch (pgs) 
    {
    case CSShutdown:
      if (CSIM_PLUGIN.quit_func != NULL) 
      {
	(*CSIM_PLUGIN.quit_func)();
	exit(1);
      }
      break;

    case CSComputeNextState:
      if (CSIM_PLUGIN.run_func) 
      {
	double_vector previous, inputs;
	double_vector answer;
	int dms;
	double tmp;

	read(from_csim, (void *)&dms, INT_SIZE);
	//assert(dms==dimensions);

	// Read the previous position
	for (int i=0; i<dms; i++) 
	{
	  read(from_csim, (void *)&tmp, DOUBLE_SIZE);
	  previous.push_back(tmp);
	}

	// Read the inputs
	for (int i=0; i<dms; i++) 
	{
	  read(from_csim, (void *)&tmp, DOUBLE_SIZE);
	  inputs.push_back(tmp);
	}

	// Get the new position and send it to CSim
	if (CSIM_PLUGIN.run_func)
	{
	  answer = (*CSIM_PLUGIN.run_func)(previous, inputs);
	}
	else
	{
	  for (int i=0; i<dms; i++)
	  {
	    answer.push_back(0.0);
	  }
	}
	for (int i=0; i<dms; i++) 
	{
	  write(to_csim, (void *)&answer[i], DOUBLE_SIZE);
	}
      }
      break;

    case CSComputeDistance:
      {
	double_vector dv1, dv2;
	double answer;
	int dms;
	double tmp;

	// Read the size of the vectors
	read(from_csim, (void *)&dms, INT_SIZE);

	// Retrieve the vectors for distance calculation
	for (int i=0; i<dms; i++)
	{
	  read(from_csim, (void *)&tmp, DOUBLE_SIZE);
	  dv1.push_back(tmp);
	}

	for (int i=0; i<dms; i++)
	{
	  read(from_csim, (void *)&tmp, DOUBLE_SIZE);
	  dv2.push_back(tmp);
	}

	// Run the plugin and get the answer
	if (CSIM_PLUGIN.distance_func)
	{
	  answer = (*CSIM_PLUGIN.distance_func)(dv1, dv2);
	}
	else
	{
	  answer = 0.0;
	}

	// Send the distance back to csim
	write(to_csim, (void *)&answer, DOUBLE_SIZE);
      }
      break;

    case CSInit:
      break;

    default:
      fprintf(stderr, "Invalid message encountered: %d\n", pgs);
      break;
    }
  }

  // All done
}
