// Uses the intersection routines described in Intro to Ray Tracing

#include "triplevec.h"
#include "polygon.h"
#include <math.h>

#define ADJUST(x)  x*500+100

Polygon::Polygon(const Polygon &p) 
{
  // Copy all vertices
  for (int i=0; i<p.numVerts; i++) 
  {
    vertices[i] = p.vertices[i];
  }

  // Copy data over
  numVerts = p.numVerts;
}

Polygon::Polygon(Point *verts, int vertCount) 
{
  // Copy all verts
  for (int i=0; i<vertCount; i++) 
  {
    vertices[i] = verts[i];
  }

  // Set data
  numVerts = vertCount;
}  

void Polygon::describe(ostream &os, bool filled=true) 
{
  if (filled)
  {
    os << " 0 0 0 setrgbcolor \n";
  }
  else
  {
    os << " 1 0 0 setrgbcolor \n";
  }

  os << ADJUST(vertices[0][0]) << " " 
     << ADJUST(vertices[0][1]) << " moveto\n";
  for (int i=1; i<numVerts; i++) 
  {
  os << ADJUST(vertices[i][0]) << " " 
     << ADJUST(vertices[i][1]) << " lineto\n";
  }
  os << " closepath \n";

  if (filled)
  {
    os << "0.6 0.6 1.0 setrgbcolor fill\n";
  }

  os << flush;
}

void Polygon::addVert(const Point &v) {
  vertices[numVerts] = v;
  numVerts++;
}

bool Polygon::intersect(const Point &r) {
  // This intersection is described in (IRT, 53)
  Point primea, primeb, ipt;
  int sh=1, nsh=1, nc=0;
  bool retVal;

  if (1==1) {
    // Check intersection with polygon bounds
    ipt = r;
    primea = vertices[0] - ipt;
    if (primea[1] < 0) {
      sh = -1;
    }

    for (int i=1; i<=numVerts; i++) {
      primeb = vertices[i%numVerts] - ipt;
      if (primeb[1] < 0) {
	nsh = -1;
      }

      if (sh != nsh) {
	if (primea[0] >= 0 && primeb[0] >= 0) {
	  nc++;
	} else if (primea[0] >=0 || primeb[0] >= 0) {
	  if (primea[0]-primea[1]*(primeb[0]-primea[0])/
	      (primeb[1]-primea[1])>0) {
	    nc++;
	  }
	}
      }

      primea = primeb;
      sh = nsh;
      nsh = 1;
    }

    if (nc%2 != 1) {
      // If the number of crossings is even, the
      // polygon didn't get intersected
      retVal = 0;
    }
    else
    {
      retVal = 1;
    } 

  } else {
    retVal = 1;
  }

  return retVal;
}

bool Polygon::intersect(Polygon *p)
{
  // For each vertex in this, intersect with p
  for (int i=0; i<numVerts; i++)
  {
    if (p->intersect(vertices[i]))
    {
      return true;
    }
  }

  // For each vertex in p, intersect with this
  for (int i=0; i<p->numVerts; i++)
  {
    if (intersect(p->vertices[i]))
    {
      return true;
    }
  }

  return false;
}

Polygon *Polygon::transform(vector< double > trans)
{
  double dx, dy, sintheta, costheta;
  double theta;
  Polygon *trans_poly = new Polygon();

  theta = trans[2]*2*M_PI;
  dx = trans[0];
  dy = trans[1];
  sintheta = sin(theta);
  costheta = cos(theta);

  for (int i=0; i<numVerts; i++)
  {
    Point new_point;
    new_point.set(vertices[i][0]*costheta - vertices[i][1]*sintheta + dx,
		  vertices[i][0]*sintheta + vertices[i][1]*costheta + dy);
    trans_poly->addVert(new_point);
  }

  return trans_poly;
}
