#include "triplevec.h"

#define MIN(A,B) (A>B) ? B : A

Point::Point(const Point &v) {
  v1 = v.v1;
  v2 = v.v2;
}

Point::Point(double a, double b) {
  v1 = a;
  v2 = b;
}

Point& Point::operator= (const Point &a) {
  v1 = a.v1;
  v2 = a.v2;
  return *this;
}

Point& Point::operator+=(const Point &a) {
  v1 += a.v1;
  v2 += a.v2;
  return *this;
}

Point& Point::operator-=(const Point &a) {
  v1 -= a.v1;
  v2 -= a.v2;
  return *this;
}

Point& Point::operator*=(const double d) {
  v1 *= d;
  v2 *= d;
  return *this;
}

double Point::operator[](int a) const {
  if (a==0) {
    return v1;
  } else if (a==1) {
    return v2;
  } else if (a==2) {
    return 1.0;
  } else {
    return 0.0;
  }
}

Point operator-(const Point &a) {
  return Point(-a.v1, -a.v2);
}

Point operator+(const Point &a, const Point &b) {
  return Point(a.v1+b.v1, a.v2+b.v2);
}

Point operator-(const Point &a, const Point &b) {
  return Point(a.v1-b.v1, a.v2-b.v2);
}

Point operator*(const Point &a, const double &d) {
  return Point(a.v1*d, a.v2*d);
}

Point operator*(const Point &a, const Point &b) {
  return Point(a.v1*b.v1, a.v2*b.v2);
}

Point operator*(const double d, const Point &a) {
  return Point(a.v1*d, a.v2*d);
}

Point operator/(const Point &a, const double d) {
  return Point(a.v1/d, a.v2/d);
}

bool operator==(const Point &a, const Point &b) {
  return (a.v1==b.v1 && a.v2==b.v2);
}

bool operator< (const Point &a, const Point &b) {
  return (a.Magnitude() < b.Magnitude());
}

bool operator> (const Point &a, const Point &b) {
  return !(a<b);
}

bool operator!=(const Point &a, const Point &b) {
  return !(a==b);
}

ostream& operator<<(ostream &out, const Point &a) {
  out << '<' << a.v1 
      << ',' << a.v2 << '>';
  return out;
}

double Point::Magnitude() const {
  return sqrt(v1*v1+v2*v2);
}

Point Point::normalize() const {
  double m = Magnitude();
  if (m==0) m=1;
  return (*this)/m;
}
