// copyright

/**
 * This is a simple plugin to test the architecture and
 * make sure that it is doing what it is supposed to.  It
 * closely mimics Model2d.
 *
 */

#include "csim.h"

void test1_quit(void);
void test1_init(void);
double test1_dist(double_vector &dv1, double_vector &dv2);
double_vector test1_run(double_vector &pre, double_vector &inputs);

CSPluginInfo CSIM_PLUGIN = 
{
  "test1_plugin",    /* name          */
  3,                 /* dimensions    */
  test1_quit,        /* quit_func     */
  test1_init,        /* init_func     */
  test1_dist,        /* distance_func */
  test1_run          /* run_func      */
};

//MAIN()

void test1_quit(void)
{
  cerr << "Received the signal to quit...\n" << flush;
}

void test1_init(void)
{
  cerr << "Received the signal to initialize...\n" << flush;
}

double test1_dist(double_vector &dv1, double_vector &dv2)
{
  double dist;
  int size = dv1.size();

  // Compute the Euclidian distance metric
  for (int i=0; i<size; i++)
  {
    double dv1_inf = dv1[i] - dv2[i];
    dist += dv1_inf * dv1_inf;
  }

  return sqrt(dist);
}

double_vector test1_run(double_vector &pre, double_vector &inputs)
{
  double dist = test1_dist(pre, inputs);
  int size = pre.size();
  double grow_amt = (dist > 0.05 ?
		     0.05 :
		     1);
  double_vector answer;

  for (int i=0; i<size-1; i++)
  {
    answer.push_back(pre[i] + grow_amt * (inputs[i] - pre[i]));
  }

  answer.push_back(0);

  return answer;
}
