// copyright

/**
 * This is a simple plugin to test the architecture and
 * make sure that it is doing what it is supposed to.  It
 * will only allow rotations of at most PI/6.
 *
 */

#include "csim.h"

#ifndef M_PI
#define M_PI 3.14159267
#endif

void test2_quit(void);
void test2_init(void);
double test2_dist(double_vector &dv1, double_vector &dv2);
double_vector test2_run(double_vector &pre, double_vector &inputs);

CSPluginInfo CSIM_PLUGIN = 
{
  "test2_plugin",    /* name          */
  3,                 /* dimensions    */
  test2_quit,        /* quit_func     */
  test2_init,        /* init_func     */
  test2_dist,        /* distance_func */
  test2_run          /* run_func      */
};

//MAIN()

void test2_quit(void)
{
  cerr << "Received the signal to quit...\n" << flush;
}

void test2_init(void)
{
  cerr << "Received the signal to initialize...\n" << flush;
}

double test2_dist(double_vector &dv1, double_vector &dv2)
{
  double dist, dv1_inf;
  int size = dv1.size()-1;

  // Compute the Euclidian distance metric
  for (int i=0; i<size; i++)
  {
    dv1_inf = dv1[i] - dv2[i];
    dist += dv1_inf * dv1_inf;
  }

  return sqrt(dist);
}

double_vector test2_run(double_vector &pre, double_vector &inputs)
{
  double dist = test2_dist(pre, inputs);
  int size = pre.size();
  double grow_amt;
  double_vector answer;
  double rot_size, x, y;
  double costheta, sintheta;

  rot_size = inputs[2] - pre[2];

  if (rot_size > 1.0)
  {
    rot_size -= 1.0;
  }
  else if (rot_size < -1.0)
  {
    rot_size += 1.0;
  }

  if (rot_size  > 0.15)
  {
    rot_size = 0.15;
  }
  else if (rot_size < -0.15)
  {
    rot_size = -0.15;
  }

  rot_size += pre[2];

  x = pre[0]-inputs[0];
  y = pre[1]-inputs[1];
  grow_amt = sqrt(x*x + y*y);
  grow_amt = (grow_amt > 0.035 ? 0.035 : x);

  costheta(rot_size*2*M_PI);
  sintheta(rot_size*2*M_PI);

  x = grow_amt * (costheta) + pre[0];
  y = grow_amt * (sintheta) + pre[1];

  answer.push_back(x);
  answer.push_back(y);
  answer.push_back(rot_size);


  return answer;
}
